@@ -2,7 +2,7 @@ import typing as T
 from copy import copy
 import inspect
 
-from .utils.json import DescriptionJSONEncoder
+from .utils.json import DescriptionJSONEncoder, DescriptionJSONDecoder
 from .utils.misc import CreateByGetItem
 
 
@@ -22,6 +22,7 @@ RangeChecker = T.Callable[[T.Any, T.Any], bool]
 
 
 class Value(metaclass=CreateByGetItem):
+    """The description of a value."""
     type_to_range_checker: T.Dict[type, TypeChecker] = {}
     type_to_type_checker: T.Dict[type, RangeChecker] = {}
 
@@ -30,7 +31,8 @@ class Value(metaclass=CreateByGetItem):
             type_: T.Optional[T.Type[T1]] = None,
             range_: T.Optional[T.Any] = None,
             default: T.Union[_NotDef, T1] = NotDef,
-            name: str = "?",
+            name: T.Optional[str] = None,
+            doc: T.Optional[str] = None,
             **kwargs,
             ):
         self.name = name
@@ -38,6 +40,7 @@ class Value(metaclass=CreateByGetItem):
         self.range = range_
         self.default = default
         self.kwargs = kwargs
+        self.doc = doc
 
     @property
     def type_checker(self):
@@ -76,10 +79,19 @@ class Value(metaclass=CreateByGetItem):
 
     def __repr__(self):
         return (
-            f"<Value type={self.type} range={self.range} "
+            f"<Value name={self.name} type={self.type} range={self.range} "
             f"default={self.default}>"
         )
 
+    def __eq__(self, other):
+        return (
+            self.name == other.name and
+            self.doc == other.doc and
+            self.type == other.type and
+            self.range == other.range and
+            self.default == other.default
+        )
+
 
 # register basic types
 def _check_number_in_range(v, range):
@@ -111,17 +123,25 @@ class SideEffect():
     def __repr__(self):
         return f"<{self.__class__.__name__} description={self.description}>"
 
+    def __eq__(self, other):
+        return self.description == other.description
+
 
 class Description():
+    """The description of a function."""
     def __init__(
             self,
             inputs: T.Optional[T.List[Value]] = None,
             outputs: T.Optional[T.List[Value]] = None,
             side_effects: T.Optional[T.List[SideEffect]] = None,
+            name: T.Optional[str] = None,
+            doc: T.Optional[str] = None,
             ):
         self.inputs = [] if inputs is None else inputs
         self.outputs = [] if outputs is None else outputs
         self.side_effects = [] if side_effects is None else side_effects
+        self.name = name
+        self.doc = doc
 
     def parse_pass_in(self, args: tuple, kwargs: dict) -> T.Dict[str, T.Any]:
         """Get the pass in value of the func
@@ -132,7 +152,7 @@ class Description():
         res = {}
         for val in self.inputs:
             has_default = val.default is not NotDef
-            name: str = val.name
+            name = "?" if val.name is None else val.name
             if len(args_) > 0:
                 res[name] = args_.pop(0)
             elif (len(kwargs) > 0) and (name in kwargs):
@@ -150,15 +170,30 @@ class Description():
         json_str = DescriptionJSONEncoder().encode(self)
         return json_str
 
+    @classmethod
+    def from_json(cls, json_str: str) -> "Description":
+        return DescriptionJSONDecoder().decode(json_str)
+
     def __repr__(self) -> str:
         return (
             "<Description\n"
             f"\tinputs={self.inputs}\n"
             f"\toutputs={self.outputs}\n"
             f"\tside_effects={self.side_effects}\n"
+            f"\tname={self.name}\n"
+            f"\tdoc={self.doc}\n"
             ">"
         )
 
+    def __eq__(self, other):
+        return (
+            self.inputs == other.inputs and
+            self.outputs == other.outputs and
+            self.side_effects == other.side_effects and
+            self.name == other.name and
+            self.doc == other.doc
+        )
+
     def compose_signature(self) -> inspect.Signature:
         """Compose the signature of the function
         according to the description."""
@@ -171,7 +206,7 @@ class Description():
                 default = val.default
             params.append(
                 inspect.Parameter(
-                    val.name,
+                    val.name or "?",
                     inspect.Parameter.POSITIONAL_OR_KEYWORD,
                     default=default,
                     annotation=val.type,
